@@ -11,7 +11,6 @@ from git.exc import InvalidGitRepositoryError
 
 import calkit
 from calkit.cli.main import _to_shell_cmd
-from calkit.core import ryaml
 
 
 def test_run_in_env(tmp_dir):
@@ -21,13 +20,11 @@ def test_run_in_env(tmp_dir):
     subprocess.check_call(
         "calkit new docker-env "
         "--name my-image "
-        "--stage build-image "
         "--from ubuntu "
         "--add-layer miniforge "
         "--description 'This is a test image'",
         shell=True,
     )
-    subprocess.check_call("calkit run", shell=True)
     out = (
         subprocess.check_output("calkit xenv echo sup", shell=True)
         .decode()
@@ -40,7 +37,6 @@ def test_run_in_env(tmp_dir):
         "calkit new docker-env "
         "-n env2 "
         "--image my-image-2 "
-        "--stage build-image-2 "
         "--path Dockerfile.2 "
         "--from ubuntu "
         "--add-layer miniforge "
@@ -48,12 +44,6 @@ def test_run_in_env(tmp_dir):
         "--description 'This is a test image 2'",
         shell=True,
     )
-    with open("dvc.yaml") as f:
-        pipeline = ryaml.load(f)
-    stg = pipeline["stages"]["build-image-2"]
-    cmd = stg["cmd"]
-    assert "-i Dockerfile.2" in cmd
-    subprocess.check_call("calkit run", shell=True)
     with pytest.raises(subprocess.CalledProcessError):
         out = (
             subprocess.check_output("calkit xenv echo sup", shell=True)
@@ -95,6 +85,26 @@ def test_run_in_env(tmp_dir):
     ck_info = calkit.load_calkit_info()
     env = ck_info["environments"]["py3.10"]
     assert env.get("path") is None
+    # Test that we can run a command that changes directory first
+    os.makedirs("my-new-dir/another", exist_ok=True)
+    out = (
+        subprocess.check_output(
+            "calkit xenv -n py3.10 --wdir my-new-dir -- ls",
+            shell=True,
+        )
+        .decode()
+        .strip()
+    )
+    assert out == "another"
+    out = (
+        subprocess.check_output(
+            "calkit xenv -n py3.10 --wdir my-new-dir -- ls ..",
+            shell=True,
+        )
+        .decode()
+        .strip()
+    )
+    assert "my-new-dir" in out.split("\n")
 
 
 def test_run_in_venv(tmp_dir):
@@ -178,7 +188,7 @@ def test_run_in_venv(tmp_dir):
     )
     ck_info = calkit.load_calkit_info(as_pydantic=True)
     envs = ck_info.environments
-    env = envs["my-pixi"]
+    assert "my-pixi" in envs
     out = (
         subprocess.check_output(
             [
@@ -216,6 +226,13 @@ def test_to_shell_cmd():
     shell_cmd = _to_shell_cmd(cmd)
     assert shell_cmd == 'python -c "print(\\"hello world\\")"'
     subprocess.check_call(shell_cmd, shell=True)
+    cmd = [
+        "sh",
+        "-c",
+        "cd dir1 && ls",
+    ]
+    good_shell_cmd = 'sh -c "cd dir1 && ls"'
+    assert _to_shell_cmd(cmd) == good_shell_cmd
 
 
 def test_add(tmp_dir):
